#ifndef __OBDNET_H
#define __OBDNET_H



#if !defined(__GNUC__)
# define __attribute__(x)
#endif



#if defined(_WIN32)

# include <windows.h>

# define OBDNET_API					__stdcall
# define OBDNET_CALLBACK			__stdcall

#elif defined(__GNUC__)

# define OBDNET_API					__attribute__((stdcall))
# define OBDNET_CALLBACK			__attribute__((stdcall))
# define TEXT(x)					x
# define BOOL						int

#endif


#ifdef __cplusplus
extern "C" {
#endif



//
// API Types:
// ==========
// C/C++ -        This API type configures the OBDNet library to operate in
//                C/C++ mode. In this mode, among other things, the OBDNet
//                library accepts and returns strings as NULL-terminated UTF-8
//                encoded "C"-style strings.
//
// Visual Basic - This API type configures the OBDNet library to operate in
//                Visual Basic mode. In this mode, among other things, the OBDNet
//                library accepts and returns strings as Win32 BSTR objects which
//                are expected by Visual Basic code.
//
//
// WARNING: ALL STRINGS RETURNED BY THIS LIBRARY ARE ONLY VALID UNTIL THE NEXT
// CALL INTO THIS LIBRARY, AT WHICH POINT THEY MAY BE OVERWRITTEN BY NEW STRING
// DATA. IF YOU NEED STRINGS TO PERSIST ACROSS LIBRARY FUNCTION CALLS THEN YOU
// MUST SAVE THE STRING DATA TO YOUR OWN STORAGE LOCATION BEFORE MAKING ANOTHER
// LIBRARY CALL.
//
#define OBDNET_APITYPE_C_CPLUSPLUS	1
#define OBDNET_APITYPE_VISUALBASIC	2



typedef unsigned long				OBDNET_STATUS;

typedef void*						OBDNET_HANDLE;

#define OBDNET_STATUS_SUCCESS		0




// Logging Modes
#define OBDNET_LOGGINGMODE_UNKNOWN				0
#define OBDNET_LOGGINGMODE_STOPPED				1
#define OBDNET_LOGGINGMODE_MONITORING			2
#define OBDNET_LOGGINGMODE_RECORDING			3



#define OBDNET_PID_VALUETYPE_NUMBER				1
#define OBDNET_PID_VALUETYPE_STRING				2

#pragma pack(push, 4)

typedef struct
{
	unsigned long	PidValueType;				// Parameter value type
	unsigned long	TimeStamp;					// The time stamp is the number of milliseconds since logging started
	double			EnglishValue;				// For OBDNET_PID_VALUETYPE_NUMBER types
	double			MetricValue;				// For OBDNET_PID_VALUETYPE_NUMBER types
	const char*		EnglishValueString;			// For OBDNET_PID_VALUETYPE_STRING types
	const char*		MetricValueString;			// For OBDNET_PID_VALUETYPE_STRING types
	const char*		UniqueId;					// Parameter Id - i.e. "SAE.RPM"
} __attribute__((aligned(4),packed)) OBDNET_PID_VALUE;

#pragma pack(pop)

// OBDNet Callbacks
#define OBDNET_CALLBACK_PID_LIST_ITEM			0
#define OBDNET_CALLBACK_SELECTED_PID_ITEM		1
#define OBDNET_CALLBACK_STREAMING_PID_VALUE		2

typedef void	(OBDNET_CALLBACK* OBDNET_CALLBACK_PID_LIST_ITEM_PROTOTYPE) (OBDNET_HANDLE OBDNetHandle, const char* lpszPidUniqueId);
typedef void	(OBDNET_CALLBACK* OBDNET_CALLBACK_SELECTED_PID_ITEM_PROTOTYPE) (OBDNET_HANDLE OBDNetHandle, const char* lpszPidUniqueId);
typedef void	(OBDNET_CALLBACK* OBDNET_CALLBACK_STREAMING_PID_VALUE_PROTOTYPE) (OBDNET_HANDLE OBDNetHandle, const OBDNET_PID_VALUE* pPidValue);


#pragma pack(push, 4)

typedef struct
{
	unsigned long	Supported;					// Zero for not-supported, non-zero for supported
	unsigned long	Priority;					// The parameters priority (can be priority 1, 2, or 3)
	const char*		UniqueId;					// Parameter Id - i.e. "SAE.RPM"
	const char*		Name;						// Parameter name i.e. Engine RPM
	const char*		Category;					// Parameter category i.e. Misfire
	const char*		EnglishUnits;				// English units i.e. MPH or RPM
	const char*		MetricUnits;				// Metric units i.e. KMH or RPM
} __attribute__((aligned(4),packed)) OBDNET_PID_INFO;

#pragma pack(pop)




// Initialization functions
OBDNET_STATUS OBDNET_API	OBDNetInitialize(unsigned long ApiType, void* lpReserved);
OBDNET_STATUS OBDNET_API	OBDNetUninitialize(void);

// Creation functions
OBDNET_HANDLE OBDNET_API	OBDNetCreate(void);
OBDNET_STATUS OBDNET_API	OBDNetDestroy(OBDNET_HANDLE OBDNetHandle);

// Status functions
const char* OBDNET_API		OBDNetStatusGetErrorCodeString(OBDNET_STATUS ErrorCode);

// Debugging functions
OBDNET_STATUS OBDNET_API	OBDNetEnableDebugMode(const char* lpszTechlogFileName, unsigned long DebugLevel, BOOL EnableDebugTracing);
OBDNET_STATUS OBDNET_API	OBDNetEnableSequentialWriting(BOOL EnableSequentialWriting);
OBDNET_STATUS OBDNET_API	OBDNetDisableDebugMode(void);

// Connection functions
OBDNET_STATUS OBDNET_API	OBDNetOpenConnection(OBDNET_HANDLE OBDNetHandle, const char* lpszServer, unsigned long Port /* Default port is 1301 */, unsigned long TimeoutInMilliseconds);
OBDNET_STATUS OBDNET_API	OBDNetCloseConnection(OBDNET_HANDLE OBDNetHandle);
BOOL OBDNET_API				OBDNetIsConnected(OBDNET_HANDLE OBDNetHandle);

// Misc. functions
OBDNET_STATUS OBDNET_API	OBDNetWorker(OBDNET_HANDLE OBDNetHandle);
OBDNET_STATUS OBDNET_API	OBDNetSetCallback(unsigned long CallbackNumber, void* pCallback);
OBDNET_STATUS OBDNET_API	OBDNetSetWorkerTimeout(OBDNET_HANDLE OBDNetHandle, unsigned long TimeoutInMilliseconds);
OBDNET_STATUS OBDNET_API	OBDNetSetFunctionCallTimeout(OBDNET_HANDLE OBDNetHandle, unsigned long TimeoutInMilliseconds);

// Parameter functions
unsigned long OBDNET_API	OBDNetParameterGetCount(OBDNET_HANDLE OBDNetHandle);
const char* OBDNET_API		OBDNetParameterGetUniqueId(OBDNET_HANDLE OBDNetHandle, unsigned long PidIndex);
OBDNET_STATUS OBDNET_API	OBDNetParameterGetPidList(OBDNET_HANDLE OBDNetHandle);
OBDNET_STATUS OBDNET_API	OBDNetParameterGetInfo(OBDNET_HANDLE OBDNetHandle, const char* lpszUniqueId, OBDNET_PID_INFO* lpPidInfo);
OBDNET_STATUS OBDNET_API	OBDNetParameterGetCurrentValue(OBDNET_HANDLE OBDNetHandle, const char* lpszUniqueId, OBDNET_PID_VALUE* lpPidValue);

// Logging functions
OBDNET_STATUS OBDNET_API	OBDNetLoggingGetSelectedPidList(OBDNET_HANDLE OBDNetHandle);
OBDNET_STATUS OBDNET_API	OBDNetLoggingClearSelectedPidList(OBDNET_HANDLE OBDNetHandle);
OBDNET_STATUS OBDNET_API	OBDNetLoggingAddSelectedPid(OBDNET_HANDLE OBDNetHandle, const char* lpszPid);
OBDNET_STATUS OBDNET_API	OBDNetLoggingRemoveSelectedPid(OBDNET_HANDLE OBDNetHandle, const char* lpszPid);
unsigned long OBDNET_API	OBDNetLoggingGetMode(OBDNET_HANDLE OBDNetHandle);
OBDNET_STATUS OBDNET_API	OBDNetLoggingSetMode(OBDNET_HANDLE OBDNetHandle, unsigned long LoggingMode);
OBDNET_STATUS OBDNET_API	OBDNetLoggingStartStreamingPid(OBDNET_HANDLE OBDNetHandle, const char* lpszPid);
OBDNET_STATUS OBDNET_API	OBDNetLoggingStopStreamingPid(OBDNET_HANDLE OBDNetHandle, const char* lpszPid);
OBDNET_STATUS OBDNET_API	OBDNetLoggingStopStreamingAllPids(OBDNET_HANDLE OBDNetHandle);




#ifdef __cplusplus
}
#endif


#endif // defined __OBDNET_H
